# terraform apply -var-file="terraform.tfvars" -auto-approve

terraform {
  required_providers {
    openstack = {
      source  = "terraform-provider-openstack/openstack"
      version = "~> 3.0.0"
    }
    aws = {
      source  = "hashicorp/aws"
      version = "~> 5.0.0"
    }
    local = {
      source  = "hashicorp/local"
      version = "~> 2.0"
    }
  }
  required_version = ">= 1.9.0"
}

# Configure the OpenStack Provider
provider "openstack" {
  cloud = "engines"
}

# SSH Key generation
resource "tls_private_key" "my_generated_key" {
  algorithm = "RSA"
  rsa_bits  = 2048
}

resource "openstack_compute_keypair_v2" "my_keypair" {
  name       = "${var.key_name}-keypair"
  public_key = tls_private_key.my_generated_key.public_key_openssh
}

resource "local_file" "private_key" {
  content  = tls_private_key.my_generated_key.private_key_pem
  filename = "${pathexpand(var.private_key_path)}/${var.key_name}-keypair.pem"

  provisioner "local-exec" {
    command = "chmod 600 ${self.filename}"
  }
}

resource "openstack_compute_instance_v2" "app_server" {
  name        = "${var.instance_name}-instance"
  image_id    = "654bf798-579b-47aa-a7f7-8a8798c9779d"
  flavor_name = "m1.medium"
  key_pair    = openstack_compute_keypair_v2.my_keypair.name

  security_groups = ["default", "${var.secgrp_name}-group"]
  user_data = data.cloudinit_config.my_config.rendered
}

# Network configuration
resource "openstack_networking_secgroup_v2" "secgrp_1" {
  name = "${var.secgrp_name}-group"
}

resource "openstack_networking_secgroup_rule_v2" "secgroup_rule_1" {
  direction         = "ingress"
  ethertype         = "IPv4"
  protocol          = "icmp"
  remote_ip_prefix  = "0.0.0.0/0"
  security_group_id = openstack_networking_secgroup_v2.secgrp_1.id
}

resource "openstack_networking_secgroup_rule_v2" "secgroup_rule_2" {
  direction         = "ingress"
  ethertype         = "IPv4"
  protocol          = "tcp"
  port_range_min    = 22
  port_range_max    = 22
  remote_ip_prefix  = "0.0.0.0/0"
  security_group_id = openstack_networking_secgroup_v2.secgrp_1.id
}

resource "openstack_networking_secgroup_rule_v2" "secgroup_rule_3" {
  direction         = "ingress"
  ethertype         = "IPv4"
  protocol          = "tcp"
  port_range_min    = 443
  port_range_max    = 443
  remote_ip_prefix  = "0.0.0.0/0"
  security_group_id = openstack_networking_secgroup_v2.secgrp_1.id
}

resource "openstack_networking_secgroup_rule_v2" "secgroup_rule_4" {
  direction         = "ingress"
  ethertype         = "IPv4"
  protocol          = "tcp"
  port_range_min    = 8080
  port_range_max    = 8080
  remote_ip_prefix  = "0.0.0.0/0"
  security_group_id = openstack_networking_secgroup_v2.secgrp_1.id
}

resource "openstack_networking_floatingip_v2" "fip_1" {
  pool        = "public"
  description = var.project_name
}

data "openstack_networking_port_v2" "port" {
  device_id  = openstack_compute_instance_v2.app_server.id
  network_id = openstack_compute_instance_v2.app_server.network.0.uuid
}

resource "openstack_networking_floatingip_associate_v2" "fip_associate" {
  floating_ip = openstack_networking_floatingip_v2.fip_1.address
  port_id     = data.openstack_networking_port_v2.port.id
}

# Cloud-init configuration
data "cloudinit_config" "my_config" {
  gzip          = false
  base64_encode = false

  part {
    filename     = "file-1"
    content_type = "text/cloud-config"
    content      = file("conf/cloud-init.users.yaml")
    merge_type   = "list(append)+dict(no_replace,recurse_list)"
  }
}

# AWS S3-compatible provider for OpenStack Swift
provider "aws" {
  region                      = "us-east-1"
  skip_credentials_validation = true
  skip_requesting_account_id  = true
  access_key                  = var.os_access_key
  secret_key                  = var.os_secret_key

  endpoints {
    s3 = "https://os.zhdk.cloud.switch.ch"
  }
}

# Bucket and object creation in OpenStack Swift
resource "aws_s3_bucket" "bucket" {
  bucket = "${var.bucket_name}-bucket"
}

resource "aws_s3_bucket_policy" "public_policy" {
  bucket = aws_s3_bucket.bucket.id

  policy = jsonencode({
    Version = "2012-10-17"
    Statement = [
      {
        Effect = "Allow"
        Principal = "*"
        Action = [
          "s3:GetObject",
          "s3:PutObject",
          "s3:DeleteObject"
        ]
        Resource = "${aws_s3_bucket.bucket.arn}/*"
      }
    ]
  })
}
